@@ -3,15 +3,15 @@
 HTTP-based ReStructuredText viewer.
 
 Usage:
-    restviewhttp [options] filename.rst
+    restview [options] filename.rst [...]
 or
-    restviewhttp [options] directory
+    restview [options] directory [...]
 or
-    restviewhttp [options] -e "command"
+    restview [options] -e "command"
 or
-    restviewhttp [options] --long-description
+    restview [options] --long-description
 or
-    restviewhttp --help
+    restview --help
 
 Needs docutils and a web browser. Will syntax-highlight code or doctest blocks
 (needs pygments).
@@ -25,8 +25,8 @@ import time
 import socket
 import optparse
 import threading
+import subprocess
 import webbrowser
-from contextlib import closing
 
 try:
     import BaseHTTPServer
@@ -65,7 +65,7 @@ except NameError:
     unicode = str
 
 
-__version__ = "2.0.3"
+__version__ = "2.0.4"
 
 
 class MyRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
@@ -182,8 +182,15 @@ class MyRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
 
     def handle_command(self, command):
         try:
-            with closing(os.popen(command)) as f:
-                return self.handle_rest_data(f.read())
+            p = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE,
+                                 stderr=subprocess.PIPE)
+            stdout, stderr = p.communicate()
+            if p.returncode != 0:
+                self.log_error("'%s' terminated with %s", command, p.returncode)
+            if stderr or not stdout:
+                return self.handle_error(command, p.returncode, stderr)
+            else:
+                return self.handle_rest_data(stdout)
         except OSError as e:
             self.log_error("%s", e)
             self.send_error(500, "Command execution failed")
@@ -199,6 +206,20 @@ class MyRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
         self.end_headers()
         return html
 
+    def handle_error(self, command, retcode, stderr):
+        html = self.server.renderer.render_exception(
+            title=command,
+            error='Returned error code %s' % retcode,
+            source=stderr or '(no output)')
+        if isinstance(html, unicode):
+            html = html.encode('UTF-8')
+        self.send_response(200)
+        self.send_header("Content-Type", "text/html; charset=UTF-8")
+        self.send_header("Content-Length", str(len(html)))
+        self.send_header("Cache-Control", "no-cache, no-store, max-age=0")
+        self.end_headers()
+        return html
+
     def collect_files(self, dirname):
         if not dirname.endswith(os.path.sep):
             dirname += os.path.sep
@@ -395,21 +416,18 @@ class RestViewer(object):
             docutils.core.publish_string(rest_input, writer=writer,
                                          settings_overrides=settings_overrides)
         except Exception as e:
-            return self.render_exception(e.__class__.__name__, str(e),
-                                         rest_input)
-        return self.get_markup(writer.output, mtime=mtime)
+            html = self.render_exception(e.__class__.__name__, str(e), rest_input)
+        else:
+            html = writer.output
+        return self.inject_ajax(html, mtime=mtime)
 
     def render_exception(self, title, error, source):
-        html = (ERROR_TEMPLATE.replace('$title', escape(title))
+        return (ERROR_TEMPLATE.replace('$title', escape(title))
                               .replace('$error', escape(error))
                               .replace('$source', escape(source)))
-        return self.get_markup(html)
 
-    def get_markup(self, markup, mtime=None):
-        if self.command is not None:
-            return markup.replace('</title>',
-                                  ' -e "' + escape(self.command) + '"</title>')
-        elif mtime is not None:
+    def inject_ajax(self, markup, mtime=None):
+        if mtime is not None:
             return markup.replace('</body>', (AJAX_STR % mtime) + '</body>')
         else:
             return markup
